==> RdbiPgSQL_1.0.8.tar.gz: RdbiPgSQL/R/ioMethods.R <==
# -*- R -*-
# $RCSfile: ioMethods.R,v $
# $Date: 2004/07/16 19:42:47 $
# $Revision: 1.6 $
# Copyright 2001, Timothy H. Keitt
# Licence: GPL

#
# Methods to implement Rdbi interface
#

dbSendQuery.PgSQL.conn <- function(conn, ...) {

  query <- paste(...)

  if (getOption("verbose")) cat(query, "\n") # Fix me

  result <- .Call("PgSQLsendQuery", conn, query, PACKAGE="RdbiPgSQL")

  class(result) <- c("PgSQL.result", "Rdbi.result")

  return(result)

}

dbGetQuery.PgSQL.conn <- function(conn, ...)
  dbGetResult(dbSendQuery(conn, ...))

dbListTables.PgSQL.conn <- function(conn, ...) {
    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("pattern" %in% names(args))
        pattern <- args$pattern
    else {
        if (length(unnamedArgs) >= 1)
            pattern <- unnamedArgs[[1]]
        else
            pattern <- NULL
    }

    if ("all" %in% names(args))
        all <- args$all
    else {
        if (length(unnamedArgs) >= 2)
            all <- unnamedArgs[[2]]
        else
            all <- FALSE
    }


  query <- paste("SELECT relname FROM pg_class WHERE relkind = 'r'")

  if (!is.null(pattern))
    query <- paste(query, "AND relname ~", single.quote(pattern))
  # match by regular expression

  if (!all)
    query <- paste(query, "AND NOT relname ~ '^pg_'")
  # match by regular expression

  query <- paste(query, "ORDER BY relname")

  tables <- dbGetQuery(conn, query)

  return(tables)

}

dbReadTable.PgSQL.conn <- function(conn, ...) {

    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("table.name" %in% names(args))
        table.name <- args$table.name
    else {
        if (length(unnamedArgs) >= 1)
            table.name <- unnamedArgs[[1]]
        else
          stop("No 'table.name' argument provided")
    }

    if ("row.names" %in% names(args))
        row.names <- args$row.names
    else {
        if (length(unnamedArgs) >= 2)
            row.names <- unnamedArgs[[2]]
        else
            row.names <- NULL
    }

    if ("col.names" %in% names(args))
        col.names <- args$col.names
    else {
        if (length(unnamedArgs) >= 3)
            col.names <- unnamedArgs[[3]]
        else
            col.names <- NULL
    }

    if ("as.is" %in% names(args))
        as.is <- args$as.is
    else {
        if (length(unnamedArgs) >= 4)
            as.is <- unnamedArgs[[4]]
        else
            as.is <- FALSE
    }

  result <- dbSendQuery(conn, "SELECT * FROM",
                        double.quote(table.name))

  out <- dbGetResult(result)

  as.is <- expand.asis(as.is, ncol(out))

  types <- dbColumnInfo(result)[["Type"]]

  for (i in 1:ncol(out)) {

    if (is.character(out[[i]]) && !as.is[i]) {

      data.col <- out[[i]]

      class(data.col) <- as.character(types[i])

      out[[i]] <- rpgsql.cast.values(data.col)

    }

  }

  if (!is.null(col.names)) {

    names(out) <- make.names(col.names, unique=T)

  }

  if (!is.null(row.names)) {

    row.names(out) <- as.character(row.names)

  } else if ("R_row_names" %in% names(out)) {

    row.names(out) <- out[["R_row_names"]]

    out[["R_row_names"]] <- NULL

  }

  return(out)

}

dbWriteTable.PgSQL.conn <- function(conn, ...) {

    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("data" %in% names(args))
        data <- args$data
    else {
        if (length(unnamedArgs) >= 1)
            data <- unnamedArgs[[1]]
        else
          stop("No 'data' argument provided")
    }

    if ("name" %in% names(args))
        name <- args$name
    else {
        if (length(unnamedArgs) >= 2)
            name <- unnamedArgs[[2]]
        else
            name <- deparse(substitute(data))
    }

    if ("no.clobber" %in% names(args))
        no.clobber <- args$no.clobber
    else {
        if (length(unnamedArgs) >= 3)
            no.clobber <- unnamedArgs[[3]]
        else
            no.clobber <- TRUE
    }

    if ("write.row.names" %in% names(args))
        write.row.names <- args$write.row.names
    else {
        if (length(unnamedArgs) >= 4)
            write.row.names <- unnamedArgs[[4]]
        else
            write.row.names <- FALSE
    }

  if (no.clobber) {
    # Bug fix by Markus Neteler. $relname appended
    if ("name" %in% dbListTables(conn)$relname)
      stop(paste("Table name", name, "exists in database"))

  } else {
    # Bug fix by Markus Neteler. $relname appended
    if ("name" %in% dbListTables(conn)$relname)
      dbSendQuery(conn, "DROP TABLE", double.quote(name))

  }

  if (!is.data.frame(data)) data <- as.data.frame(data)

  if (write.row.names) data[["R_row_names"]] <- row.names(data)

  query <- paste("CREATE TABLE", double.quote(name))

  column.names <- lapply(names(data), double.quote)

  data.types <- lapply(data, rpgsql.data.type)

  table.columns <- vector(mode = "character")

  for (col in 1:ncol(data)) {

    column.def <- paste(column.names[col], data.types[col])

    table.columns <- append(table.columns, column.def)

  }

  query <- paste(query, "(", list.to.csv(table.columns), ")")

  on.exit(dbSendQuery(conn, "DROP TABLE", double.quote(name)))

  dbSendQuery(conn, query)
  # create the table

  dbAppendTable(conn, name, data)
  # fill the table

  on.exit(dbSendQuery(conn, "VACUUM ANALYZE", double.quote(name)), add = TRUE)

  return(invisible())

}

dbAppendTable.PgSQL.conn <- function(conn, ...) {
  # Bug fix by Markus Neteler. $relname appended

    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("name" %in% names(args))
        name <- args$name
    else
        name <- unnamedArgs[[1]]

    if ("data" %in% names(args))
        data <- args$data
    else
        data <- unnamedArgs[[2]]

    if (!name %in% dbListTables(conn)$relname)
    stop(paste("Table", name, "does not exist"))

  if (!is.data.frame(data)) data <- as.data.frame(data)

  dbSendQuery(conn, "BEGIN TRANSACTION")

  on.exit(dbSendQuery(conn, "ROLLBACK TRANSACTION"))

  preamble <- paste("INSERT INTO", double.quote(name))

  column.names <- sapply(names(data), double.quote)

  formatted.data <- matrix(nrow=nrow(data), ncol=ncol(data))
  # this is necessary because if columns are converted to strings,
  # they will get coerced to factors (I'm sure there is a way around
  # this; I just don't know what it is.)

  for (i in seq(along = column.names))
    formatted.data[,i] <- rpgsql.format.values(data[[i]])

  boiler <- paste(preamble, "(", list.to.csv(column.names), ")")

  for (i in 1:nrow(data)) {

    insert.values <- list.to.csv(formatted.data[i,])

    dbSendQuery(conn, boiler, "VALUES (", insert.values, ")")

  }

  on.exit(dbSendQuery(conn, "COMMIT TRANSACTION"), add = TRUE)

  return(invisible())

}

==> RdbiPgSQL_1.8.0.tar.gz: RdbiPgSQL/R/ioMethods.R <==
# -*- R -*-
# $RCSfile: ioMethods.R,v $
# $Date: 2005-01-04 07:09:38 -0800 (Tue, 04 Jan 2005) $
# $Revision: 1.7 $
# Copyright 2001, Timothy H. Keitt
# Licence: GPL

#
# Methods to implement Rdbi interface
#

dbSendQuery.PgSQL.conn <- function(conn, ...) {

  query <- paste(...)

  if (getOption("verbose")) cat(query, "\n") # Fix me

  result <- .Call("PgSQLsendQuery", conn, query, PACKAGE="RdbiPgSQL")

  class(result) <- c("PgSQL.result", "Rdbi.result")

  return(result)

}

dbGetQuery.PgSQL.conn <- function(conn, ...)
  dbGetResult(dbSendQuery(conn, ...))

dbListTables.PgSQL.conn <- function(conn, ...) {
    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("pattern" %in% names(args))
        pattern <- args$pattern
    else {
        if (length(unnamedArgs) >= 1)
            pattern <- unnamedArgs[[1]]
        else
            pattern <- NULL
    }

    if ("all" %in% names(args))
        all <- args$all
    else {
        if (length(unnamedArgs) >= 2)
            all <- unnamedArgs[[2]]
        else
            all <- FALSE
    }


  query <- paste("SELECT relname FROM pg_class WHERE relkind = 'r'")

  if (!is.null(pattern))
    query <- paste(query, "AND relname ~", single.quote(pattern))
  # match by regular expression

  if (!all)
    query <- paste(query, "AND NOT relname ~ '^pg_'")
  # match by regular expression

  query <- paste(query, "ORDER BY relname")

  tables <- dbGetQuery(conn, query)

  return(tables)

}

dbReadTable.PgSQL.conn <- function(conn, ...) {

    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("table.name" %in% names(args))
        table.name <- args$table.name
    else {
        if (length(unnamedArgs) >= 1)
            table.name <- unnamedArgs[[1]]
        else
          stop("No 'table.name' argument provided")
    }

    if ("row.names" %in% names(args))
        row.names <- args$row.names
    else {
        if (length(unnamedArgs) >= 2)
            row.names <- unnamedArgs[[2]]
        else
            row.names <- NULL
    }

    if ("col.names" %in% names(args))
        col.names <- args$col.names
    else {
        if (length(unnamedArgs) >= 3)
            col.names <- unnamedArgs[[3]]
        else
            col.names <- NULL
    }

    if ("as.is" %in% names(args))
        as.is <- args$as.is
    else {
        if (length(unnamedArgs) >= 4)
            as.is <- unnamedArgs[[4]]
        else
            as.is <- FALSE
    }

    if ("as.matrix" %in% names(args))
        as.matrix <- args$as.matrix
    else
        as.matrix <- FALSE

  result <- dbSendQuery(conn, "SELECT * FROM",
                        double.quote(table.name))

  out <- dbGetResult(result, as.matrix=as.matrix)

  as.is <- expand.asis(as.is, ncol(out))

  types <- dbColumnInfo(result)[["Type"]]

    if (!as.matrix) {
        for (i in 1:ncol(out)) {

            if (is.character(out[[i]]) && !as.is[i]) {

                data.col <- out[[i]]

                class(data.col) <- as.character(types[i])

                out[[i]] <- rpgsql.cast.values(data.col)

            }

        }

        if (!is.null(col.names)) {

            names(out) <- make.names(col.names, unique=T)

        }

        if (!is.null(row.names)) {

            row.names(out) <- as.character(row.names)

        } else if ("R_row_names" %in% names(out)) {

            row.names(out) <- out[["R_row_names"]]

            out[["R_row_names"]] <- NULL

        }

    }
    return(out)
}

dbWriteTable.PgSQL.conn <- function(conn, ...) {

    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("data" %in% names(args))
        data <- args$data
    else {
        if (length(unnamedArgs) >= 1)
            data <- unnamedArgs[[1]]
        else
          stop("No 'data' argument provided")
    }

    if ("name" %in% names(args))
        name <- args$name
    else {
        if (length(unnamedArgs) >= 2)
            name <- unnamedArgs[[2]]
        else
            name <- deparse(substitute(data))
    }

    if ("no.clobber" %in% names(args))
        no.clobber <- args$no.clobber
    else {
        if (length(unnamedArgs) >= 3)
            no.clobber <- unnamedArgs[[3]]
        else
            no.clobber <- TRUE
    }

    if ("write.row.names" %in% names(args))
        write.row.names <- args$write.row.names
    else {
        if (length(unnamedArgs) >= 4)
            write.row.names <- unnamedArgs[[4]]
        else
            write.row.names <- FALSE
    }

  if (no.clobber) {
      ## FIXME: JG - I *seriously* doubt that name is uspposed to be
      ##             quoted, but will look at this later.
    # Bug fix by Markus Neteler. $relname appended
    if ("name" %in% dbListTables(conn)$relname)
      stop(paste("Table name", name, "exists in database"))

  } else {
    # Bug fix by Markus Neteler. $relname appended
    if ("name" %in% dbListTables(conn)$relname)
      dbSendQuery(conn, "DROP TABLE", double.quote(name))

  }

##  if (!is.data.frame(data)) data <- as.data.frame(data)

  if (write.row.names) data[["R_row_names"]] <- row.names(data)

  query <- paste("CREATE TABLE", double.quote(name))

  column.names <- lapply(colnames(data), double.quote)

  data.types <- lapply(data, rpgsql.data.type)

  table.columns <- vector(mode = "character")

  for (col in 1:ncol(data)) {

    column.def <- paste(column.names[col], data.types[col])

    table.columns <- append(table.columns, column.def)

  }

  query <- paste(query, "(", list.to.csv(table.columns), ")")

  on.exit(dbSendQuery(conn, "DROP TABLE", double.quote(name)))

  dbSendQuery(conn, query)
  # create the table

  dbAppendTable(conn, name, data)
  # fill the table

  on.exit(dbSendQuery(conn, "VACUUM ANALYZE", double.quote(name)))

  return(invisible())

}

dbAppendTable.PgSQL.conn <- function(conn, ...) {
  # Bug fix by Markus Neteler. $relname appended

    args <- list(...)

    if(length(names(args)) == 0){ # Added by JZ
        unnamedArgs <- args # Added by JZ
    }else{ # Added by JZ
        unnamedArgs <- args[names(args) == ""]
    } # Added by JZ

    # JZ put quotes around the names for potential arguments
    if ("name" %in% names(args))
        name <- args$name
    else
        name <- unnamedArgs[[1]]

    if ("data" %in% names(args))
        data <- args$data
    else
        data <- unnamedArgs[[2]]

    if (!name %in% dbListTables(conn)$relname)
    stop(paste("Table", name, "does not exist"))

##  if (!is.data.frame(data)) data <- as.data.frame(data)

  dbSendQuery(conn, "BEGIN TRANSACTION")

  on.exit(dbSendQuery(conn, "ROLLBACK TRANSACTION"))

  preamble <- paste("INSERT INTO", double.quote(name))

  column.names <- sapply(colnames(data), double.quote)

  formatted.data <- matrix(nrow=nrow(data), ncol=ncol(data))
  # this is necessary because if columns are converted to strings,
  # they will get coerced to factors (I'm sure there is a way around
  # this; I just don't know what it is.)

  for (i in seq(along = column.names))
    formatted.data[,i] <- rpgsql.format.values(data[[i]])

  boiler <- paste(preamble, "(", list.to.csv(column.names), ")")

  for (i in 1:nrow(data)) {

    insert.values <- list.to.csv(formatted.data[i,])

    dbSendQuery(conn, boiler, "VALUES (", insert.values, ")")

  }

  on.exit(dbSendQuery(conn, "COMMIT TRANSACTION"))

  return(invisible())

}
